
#  ThreadTestQt.py

#  http://www.naturalprogramming.com

#  2009-11-23  First program version created by Kari Laitinen
#  2009-11-23  Last modification.

#  This program was copied from

#  http://diotavelli.net/PyQtWiki/Threading,_Signals_and_Slots



import math, random, sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *

class Window( QWidget ) :

    def __init__( self, parent = None ) :
    
        QWidget.__init__(self, parent)
        
        self.thread = Worker()

        label = QLabel(self.tr("Number of stars:"))

        self.spinbox_for_star_count = QSpinBox()
        self.spinbox_for_star_count.setMaximum( 10000 )
        self.spinbox_for_star_count.setValue( 9999 )

        self.start_button = QPushButton(self.tr("&Start"))
        self.picture_area = QLabel()
        self.picture_area.setFixedSize(300, 300)

        self.connect(self.thread, SIGNAL("finished()"), self.updateUi)
        self.connect(self.thread, SIGNAL("terminated()"), self.updateUi)

        self.connect( self.thread,
                      SIGNAL( "output(QRect, QImage)" ),
                      self.add_new_star_image )


        self.connect( self.start_button,
                      SIGNAL( "clicked()" ),
                      self.activate_picture_creation )

        layout = QGridLayout()

        layout.addWidget(label, 0, 0)
        layout.addWidget(self.spinbox_for_star_count, 0, 1)
        layout.addWidget(self.start_button, 0, 2)
        layout.addWidget(self.picture_area, 1, 0, 1, 3)
        self.setLayout(layout)
        
        self.setWindowTitle(self.tr("Simple Threading Example"))

    def activate_picture_creation( self ) :
    
        self.spinbox_for_star_count.setReadOnly( True )
        self.start_button.setEnabled( False )

        pixmap_for_stars = QPixmap( self.picture_area.size() )
        pixmap_for_stars.fill(Qt.black)
        self.picture_area.setPixmap( pixmap_for_stars )

        self.thread.render( self.picture_area.size(),
                            self.spinbox_for_star_count.value() )

    def add_new_star_image( self, given_image_rectangle, given_image ) :
    
        pixmap = self.picture_area.pixmap()
        painter = QPainter()
        painter.begin(pixmap)
        painter.drawImage( given_image_rectangle, given_image )
        painter.end()
        self.picture_area.update( given_image_rectangle )

    def updateUi(self):
    
        self.spinbox_for_star_count.setReadOnly( False )
        self.start_button.setEnabled(True)

#  The worker thread is implemented as a PyQt thread rather
#  than a Python thread since we want to take advantage of the
#  signals and slots mechanism to communicate with the main application.

class Worker( QThread ) :

    def __init__(self, parent = None):
    
        QThread.__init__(self, parent)

        self.exiting = False
        self.size = QSize(0, 0)
        self.stars = 0

        self.star_path = QPainterPath()
        angle = 2*math.pi/5
        self.outerRadius = 20
        self.innerRadius = 8
        
        self.star_path.moveTo(self.outerRadius, 0)
        for step in range(1, 6):
            self.star_path.lineTo(
                self.innerRadius * math.cos((step - 0.5) * angle),
                self.innerRadius * math.sin((step - 0.5) * angle)
                )
            self.star_path.lineTo(
                self.outerRadius * math.cos(step * angle),
                self.outerRadius * math.sin(step * angle)
                )
        self.star_path.closeSubpath()

    def __del__(self):
    
        self.exiting = True
        self.wait()

    def render(self, size, stars):
    
        self.size = size
        self.stars = stars
        self.start()

    def run(self):
        
        # Note: This is never called directly. It is called by Qt once the
        # thread environment has been set up.
        
        random.seed()
        number_of_stars_to_draw = self.stars
        width = self.size.width()
        height = self.size.height()

        while not self.exiting and number_of_stars_to_draw > 0:
        
            image = QImage(self.outerRadius * 2, self.outerRadius * 2,
                           QImage.Format_ARGB32)
            image.fill(qRgba(0, 0, 0, 0))
            
            x = random.randrange(0, width)
            y = random.randrange(0, height)
            angle = random.randrange(0, 360)
            red = random.randrange(0, 256)
            green = random.randrange(0, 256)
            blue = random.randrange(0, 256)
            alpha = random.randrange(0, 256)
            
            painter = QPainter()
            painter.begin(image)
            painter.setRenderHint(QPainter.Antialiasing)
            painter.setPen(Qt.NoPen)
            painter.setBrush(QColor(red, green, blue, alpha))
            painter.translate(self.outerRadius, self.outerRadius)
            painter.rotate(angle)

            painter.drawPath( self.star_path )
            painter.end()

            self.emit(SIGNAL("output(QRect, QImage)"),
                      QRect(x - self.outerRadius, y - self.outerRadius,
                            self.outerRadius * 2, self.outerRadius * 2), image)
            number_of_stars_to_draw -= 1

if __name__ == "__main__":

    app = QApplication(sys.argv)
    window = Window()
    window.show()
    sys.exit(app.exec_())
